@@ -1,8 +1,9 @@
 import pandas as pd
 from sqlalchemy import create_engine
+import os
 
 class da_tran_SQL :
-    """interact with SQL : Update 2021-07-06
+    """interact with SQL : Update 2021-09-02
 
     :sql_type = string - type of database, support MSSQL, MYSQL, POSTGRESQL, SQLITE, BIGQUERY
 
@@ -26,7 +27,7 @@ class da_tran_SQL :
     """
     def __init__(self, sql_type, host_name, database_name, user = '', password = '' , credentials_path = None
                 , chunksize = 150, partition_size = 5000, parallel_dump = False, max_parallel = 2, **kwargs):
-        """Create connection to SQL Server"""
+        """Create connection to SQL"""
         sql_type = sql_type.upper()
         self.sql_type = sql_type
         self.chunksize = int(chunksize)
@@ -68,17 +69,21 @@ class da_tran_SQL :
                                                             ,database_name , additional_param)
         
         if (credentials_path != '') & (sql_type == 'BIGQUERY') :
-            # self.engine = create_engine(connection_str, credentials_path = credentials_path)
             from google.oauth2 import service_account
             self.project_id = host_name
             # self.dataset = self.begin_name + database_name + self.end_name + '.'
             self.dataset = database_name + '.'
-            self.engine = create_engine(connection_str)
-            if credentials_path == None : self.credentials = None
-            else : self.credentials = service_account.Credentials.from_service_account_file(credentials_path)
+            if credentials_path == None : 
+                self.credentials, self.credentials_path = None, None
+                self.engine = create_engine(connection_str)
+            else : 
+                self.engine = create_engine(connection_str, credentials_path = credentials_path)
+                self.credentials = service_account.Credentials.from_service_account_file(credentials_path)
+                self.credentials_path = credentials_path
             print(pd.read_gbq("""SELECT 'Connection OK'""",project_id = self.project_id,credentials = self.credentials).iloc[0,0]) 
         else :
             self.credentials = ''
+            self.credentials_path  = None
             self.dataset = ''
             self.engine = create_engine(connection_str)
             print(pd.read_sql_query("""SELECT 'Connection OK'""", con = self.engine).iloc[0,0]) 
@@ -86,15 +91,15 @@ class da_tran_SQL :
     def sub_dump(self, df_in,table_name_in,mode_in) :
         """normal to_sql for dask's delayed in dump_main"""
         if (self.credentials != '') & (self.sql_type == 'BIGQUERY') :
-            # print('{}{}'.format(self.dataset , table_name_in))
+            # print('{}{}'.format(self.dataset , table_name_in)) # For Debug
             df_in.to_gbq('{}{}'.format(self.dataset , table_name_in),project_id = self.project_id
-                                        ,credentials = self.credentials , reauth = True, if_exists = mode_in)
+                                        ,credentials = self.credentials , if_exists = mode_in) #, reauth = True
         else :
             df_in.to_sql(table_name_in, con = self.engine, index = False,if_exists = mode_in,chunksize = self.chunksize, method = self.method)
         return len(df_in)
 
     def dump_main(self, df_in, table_name_in ,mode_in) :
-        """Divide and Dump Dataframe"""
+        """Divide and Dump Dataframe Into Database"""
         if len(df_in) <= self.partition_size : sum_len = self.sub_dump(df_in,table_name_in,mode_in) 
         else :
             if self.parallel_dump : 
@@ -116,8 +121,14 @@ class da_tran_SQL :
         return sum_len
 
 
-    def read(self, table_name_in, condition_in = '', SP = False,raw = False, param = ''):
+    def read(self, table_name_in, condition_in = '', SP = False,raw = False, param = '', columns_list = []):
         """Read Table or View or Store Procedure"""
+        columns_read = ''
+        if len(columns_list) == 0 :columns_read = '*'
+        else : 
+            for i in columns_list :
+                if columns_read != '' : columns_read += ', '
+                columns_read += '{}{}{}.{}{}{}'.format(self.begin_name,table_name_in,self.end_name  ,  self.begin_name,i,self.end_name)
         if SP :
             if self.sql_type == 'POSTGRESQL' : return 'SP/Function None Avaliable for PostgreSQL, right now'
             sql_q = self.call_SP + ' ' + self.dataset + table_name_in + ' '
@@ -141,7 +152,7 @@ class da_tran_SQL :
                     n += 1
         elif raw : sql_q = str(table_name_in)
         else :
-            sql_q = """SELECT * FROM {}{}{}{}""".format(self.dataset,self.begin_name,table_name_in,self.end_name)
+            sql_q = """SELECT {} FROM {}{}{}{}""".format(columns_read, self.dataset,self.begin_name,table_name_in,self.end_name)
             if not condition_in == '' :
                 sql_q += """ WHERE """ + condition_in
         if (self.credentials != '') & (self.sql_type == 'BIGQUERY') :
@@ -151,7 +162,7 @@ class da_tran_SQL :
 
     def write_logic_sql(self, key , value_in):
         """Write SQL Condition Query that include >, <, ="""
-        #logic_query = self.begin_name+ str(key) + self.end_name + ' ' + value_in['logic'] + ' '
+        #logic_query = self.begin_name+ str(key) + self.end_name + ' ' + value_in['logic'] + ' ' # For Debug
         value_in['type'] = value_in.get('type','')
         if value_in.get('value','there is no value') != 'there is no value' :
             if ('date' in value_in['type']) | ('time' in value_in['type']):  # Datetime will need '' in SQL Statement
@@ -159,7 +170,6 @@ class da_tran_SQL :
                 logic_query += """'""" + str(value_in['value']) + """'"""
             else : 
                 logic_query = self.begin_name+ str(key) + self.end_name + ' ' + value_in['logic'] + ' ' + str(value_in['value'])
-#                logic_query += 
         else : raise Exception("Please insert value data")  # will get error if not specific value
         return logic_query
 
@@ -168,9 +178,11 @@ class da_tran_SQL :
         if table_name_in in list(self.engine.table_names()) :
             if fix_table : 
                 print('Delete Existing data from Table at ',pd.Timestamp.now())
+                if self.credentials_path != None : os.environ["GOOGLE_APPLICATION_CREDENTIALS"]= self.credentials_path
                 self.engine.execute("""DELETE FROM {}{}{}""".format(self.begin_name,table_name_in,self.end_name))
             else :
                 print('Drop Existing Table at ',pd.Timestamp.now())
+                if self.credentials_path != None : os.environ["GOOGLE_APPLICATION_CREDENTIALS"]= self.credentials_path
                 self.engine.execute("""DROP TABLE {}{}{}""".format(self.begin_name,table_name_in,self.end_name))
         else :
             print('Do not have table to delete at',pd.Timestamp.now())
@@ -200,13 +212,12 @@ class da_tran_SQL :
         else : logic_query = '' # Return Nothing
         #print(logic_query)
         if "'Will BE rEpLaCe wItH NULL'" in logic_query :
-            #print(logic_query)
+            #print(logic_query) # For Debug
             logic_query = logic_query.replace(", 'Will BE rEpLaCe wItH NULL'",'')
-            #print(logic_query)
+            #print(logic_query) # For Debug
             logic_query = logic_query.replace("'Will BE rEpLaCe wItH NULL',",'')
             logic_query = logic_query.replace("'Will BE rEpLaCe wItH NULL'",'')
             logic_query = '(' + logic_query + ' OR {}{}{} IS NULL'.format(self.begin_name , key, self.end_name) + ')'
-
         return logic_query
 
     def delete_old_data(self, df_in, table_name_in, list_key, math_logic, debug) :
@@ -240,25 +251,29 @@ class da_tran_SQL :
         if table_name_in in list(self.engine.table_names()) :
             print('Start delete old data at',pd.Timestamp.now())
             if debug : print(sql_q)
+            if self.credentials_path != None : os.environ["GOOGLE_APPLICATION_CREDENTIALS"]= self.credentials_path
             self.engine.execute(sql_q)
             print('Delete Last '+str(list_key)+' at',pd.Timestamp.now())
         else :
             print('Do not have table to delete at',pd.Timestamp.now())
 
-    def dump_replace(self, df_in, table_name_in, list_key, math_logic = '', partition_delete = 100000, debug = False):
+    def dump_replace(self, df_in, table_name_in, list_key, math_logic = '', partition_delete = 50000, debug = False):
         """Delete exists row of table in database with same key(s) as df and dump df append to table"""
-        if len(df_in) <= partition_delete : 
-            self.delete_old_data(df_in, table_name_in, list_key, math_logic, debug)
+        if len(df_in) == 0 : 
+            print('No Data to dump into',table_name_in,' End ',pd.Timestamp.now())
         else :
-            i, df_length = 0, len(df_in)
-            while i < df_length :
-                i_next = i + partition_delete
-                print("Processing {}-{} row from {} rows".format(i + 1, i_next, df_length))
-                self.delete_old_data(df_in.iloc[i:i_next,:],table_name_in, list_key, math_logic, debug)
-                i += partition_delete
-        #Dump df_in append to database
-        self.dump_main(df_in, table_name_in ,mode_in = 'append')
-        print('Dump data to ',table_name_in,' End ',pd.Timestamp.now())
+            if len(df_in) <= partition_delete : 
+                self.delete_old_data(df_in, table_name_in, list_key, math_logic, debug)
+            else :
+                i, df_length = 0, len(df_in)
+                while i < df_length :
+                    i_next = i + partition_delete
+                    print("Processing {}-{} row from {} rows".format(i + 1, i_next, df_length))
+                    self.delete_old_data(df_in.iloc[i:i_next,:],table_name_in, list_key, math_logic, debug)
+                    i += partition_delete
+            #Dump df_in append to database
+            self.dump_main(df_in, table_name_in ,mode_in = 'append')
+            print('Dump data to ',table_name_in,' End ',pd.Timestamp.now())
 
     def dump_new(self, df_in, table_name_in, list_key , debug = False) :
         """Delete exists row of df that has same key(s) as table and dump df_in append to table"""
@@ -311,41 +326,4 @@ class da_tran_SQL :
         #Dump df_in append to database
         print('Dump data to ',table_name_in,' Begin ',pd.Timestamp.now())
         self.dump_main( df_out, table_name_in ,mode_in = 'append') 
-        print('Dump data to ',table_name_in,' End ', pd.Timestamp.now())
-
-    def dump_new_old(self, df_in, table_name_in, list_key) :
-        """Delete exists row of df that has same key(s) as table and dump df_in append to table"""
-        print('Start Filter Existing data from df at ',pd.Timestamp.now())
-
-        #for single key
-        if 'str' in str(type(list_key))  :
-            if not list_key in df_in.columns :
-                raise Exception("{} is not in df's columns".format(list_key))       
-            sql_q = 'select distinct '+ self.begin_name +  list_key + self.end_name +' from ' + table_name_in
-            filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
-            logic_filter = ~df_in[list_key].isin(filter_filter)
-
-        #for multi keys    
-        elif 'list' in str(type(list_key)) :
-            if not min(x in df_in.columns for x in list_key) :
-                raise Exception("Some keys are not in df's columns")
-            i = 0
-            while i < len(list_key) :
-                filter_name = list_key[i]
-                sql_q = 'select distinct '+ self.begin_name +  filter_name + self.end_name +' from ' + table_name_in
-                filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
-                if i == 0 : logic_filter = df_in[filter_name].isin(filter_filter)
-                else : logic_filter = (logic_filter) & (df_in[filter_name].isin(filter_filter))
-                i += 1
-
-            logic_filter = ~logic_filter
-
-        #if key is not either string or list
-        else :
-            raise Exception("List of Key must be string or list")
-
-        df_in = df_in[logic_filter]
-
-        #Dump df_in append to database
-        self.dump_main(df_in, table_name_in ,mode_in = 'append')
         print('Dump data to ',table_name_in,' End ', pd.Timestamp.now())
\ No newline at end of file
